'use strict';

exports.__esModule = true;
exports.delimiterBuffer = exports.delimiter = exports.NextionProtocol = exports.nextionProtocol = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.read = read;

var _binProtocol = require('bin-protocol');

var _codes = require('./codes');

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @ignore
 */
var debug = (0, _debug2.default)('nextion:protocol');

/**
 * @ignore
 * @todo document
 */
var NextionProtocolClass = (0, _binProtocol.createProtocol)(function () {
  // this is mostly here for the convenience of a 3p consumer.
  var reset = this.reader.reset;
  this.reader.reset = function (buf) {
    if (buf.slice(buf.length - 3).equals(delimiterBuffer)) {
      return reset.call(this, buf.slice(0, -3));
    }
    return reset.call(this, buf);
  };
});

/**
 * A [bin-protocol](https://npmjs.com/package/bin-protocol) `Protocol` class
 * which decodes Nextion-speak.
 * Does not handle response codes!
 * @see https://npmjs.com/package/bin-protocol
 * @example
 * const protocolInstance = new NextionProtocol();
 */

var NextionProtocol =
/**
 * Constructs a {@link NextionProtocol} instance.
 */
function NextionProtocol() {
  _classCallCheck(this, NextionProtocol);

  return new NextionProtocolClass();
};

/**
 * Namespace of "read" functions which are supported by {@link NextionProtocol}.
 * @todo Document the shape of the event data
 * @private
 */


var readers = {
  byte(name) {
    this.UInt8(name);
  },
  eventCode() {
    this.byte('code');
    return _codes.eventCodeMap.get(String(this.context.code));
  },
  responseCode() {
    this.byte('code');
    return _codes.responseCodeMap.get(String(this.context.code));
  },
  touchEvent() {
    this.pageId().byte('buttonId').byte('releaseEvent');
    this.context.releaseEvent = Boolean(this.context.releaseEvent);
  },
  pageId() {
    this.byte('pageId');
  },
  touchCoordinate() {
    this.byte('xHigh').byte('xLow').byte('yHigh').byte('yLow').byte('releaseEvent');
    this.context.releaseEvent = Boolean(this.context.releaseEvent);
  },
  touchCoordinateOnWake() {
    this.touchCoordinate();
  },
  stringData() {
    this.context.value = this.buffer.toString();
  },
  numericData() {
    this.Int16LE('value');
  }
};

Object.keys(readers).forEach(function (name) {
  NextionProtocolClass.define(name, {
    read: readers[name]
  });
});

/**
 * Generic response or event from Nextion device.
 * @public
 * @abstract
 * @example
 * const result = new Result(0x01); // success
 */

var Result = function () {
  /**
   * Creates a {@link Result}.
   * @param {number} code - Decimal instruction code
   */
  function Result(code) {
    _classCallCheck(this, Result);

    /**
     * Decimal representation of instruction code.
     * @type {number}
     */
    this.code = code;
  }

  /**
   * Hexadecimal representation of instruction code.
   * @type {string}
   */


  _createClass(Result, [{
    key: 'hex',
    get: function get() {
      return (0, _util.toHex)(this.code);
    }
  }]);

  return Result;
}();

/**
 * An "event" from a Nextion device.
 * @public
 */


var EventResult = function (_Result) {
  _inherits(EventResult, _Result);

  /**
   * Creates an EventResult.
   * @param {number} code - Decimal instruction code
   * @param {*} [data] - Any other data returned by result
   */
  function EventResult(code, data) {
    _classCallCheck(this, EventResult);

    /**
     * Human-readable short name of this event.
     * @type {string}
     */
    var _this = _possibleConstructorReturn(this, _Result.call(this, code));

    _this.name = _codes.eventCodeMap.get(String(code));

    /**
     * Any other data returned by result
     * @type {*|void}
     */
    _this.data = data;
    return _this;
  }

  return EventResult;
}(Result);

/**
 * A response, either success or an error, from a command.
 * @public
 */


var ResponseResult = function (_Result2) {
  _inherits(ResponseResult, _Result2);

  /**
   * Creates a ResponseResult.
   * @param {number} code - Decimal instruction code
   */
  function ResponseResult(code) {
    _classCallCheck(this, ResponseResult);

    /**
     * Human-readable short name of this response.
     * @type {string}
     */
    var _this2 = _possibleConstructorReturn(this, _Result2.call(this, code));

    _this2.name = _codes.responseCodeMap.get(String(code));
    return _this2;
  }

  return ResponseResult;
}(Result);

/**
 * Parse an event or response from a Nextion device.
 * @param {Buffer} buf - Raw `Buffer` from Nextion device
 * @returns {Result} - Parsed result
 * @throws {TypeError} When unknown data received from device
 * @private
 */


function read(buf) {
  var code = buf.readUInt8(0);
  debug('Code:', (0, _util.toHex)(code));
  if (_codes.responseCodeMap.has(String(code))) {
    return new ResponseResult(code);
  }

  if (_codes.eventCodeMap.has(String(code))) {
    var eventResult = new EventResult(code);
    var reader = nextionProtocol.read(buf.slice(1));
    if (_fp2.default.isFunction(reader[eventResult.name])) {
      eventResult.data = reader[eventResult.name]().result;
    }
    return eventResult;
  }

  throw new TypeError(`Unknown data received: ${buf.toString()}`);
}

/**
 * This instance is used internally.
 * @type {NextionProtocol}
 * @private
 */
var nextionProtocol = exports.nextionProtocol = new NextionProtocol();

exports.NextionProtocol = NextionProtocolClass;

/**
 * This is the byte delimiter all Nextion data ends with.
 * @type {number[]}
 * @private
 */

var delimiter = exports.delimiter = [0xff, 0xff, 0xff];

/**
 * Buffer wrapper of {@link delimiter}.
 * @private
 * @type {Buffer}
 */
var delimiterBuffer = exports.delimiterBuffer = Buffer.from(delimiter);
//# sourceMappingURL=protocol.js.map